<?xml version="1.0"?>

<st-source>
<time-stamp>From VisualWorks®, 7.7 of 16. November 2009 on 9. Januar 2010 at 15:41:33</time-stamp>


<component-created>
<name>TranslationScanner</name> <type>package</type>
</component-created><!-- Package TranslationScanner -->


<class>
<name>Entry</name>
<environment>Smalltalk</environment>
<super>Core.Object</super>
<private>false</private>
<indexed-type>none</indexed-type>
<inst-vars>id key translation comment </inst-vars>
<class-inst-vars></class-inst-vars>
<imports></imports>
<category></category>
<attributes>
<package>TranslationScanner</package>
</attributes>
</class>

<comment>
<class-id>Entry</class-id>
<body>Entry represents on translation in the strings-file

Instance Variables
	comment	&lt;String&gt;		the comment
	id			&lt;String&gt;		the objectId
	key			&lt;String&gt;		the lookup key
	translation	&lt;String&gt;		the translation

</body>
</comment>

<class>
<name>Converter</name>
<environment>Smalltalk</environment>
<super>Core.Object</super>
<private>false</private>
<indexed-type>none</indexed-type>
<inst-vars>originalEntries missedIds </inst-vars>
<class-inst-vars></class-inst-vars>
<imports></imports>
<category></category>
<attributes>
<package>TranslationScanner</package>
</attributes>
</class>

<comment>
<class-id>Converter</class-id>
<body>Converter converts Strings files.

Usage

Converter new convert: '/Path/To/Project/German.lproj'
Converter new filter: '/Path/To/Project/German.lproj'

Instance Variables
	missedIds	&lt;Collection&gt;	description of missedIds
	originalEntries	&lt;Dictionary&gt;	description of originalEntries

</body>
</comment>

<methods>
<class-id>Converter class</class-id> <category>instance creation</category>

<body package="TranslationScanner" selector="new">new
	"Answer a newly created and initialized instance."

	^super new initialize</body>
</methods>


<methods>
<class-id>Converter</class-id> <category>initialize-release</category>

<body package="TranslationScanner" selector="filterFrom:to:">filterFrom: source to: target

	| rs ws entry entries currentEntries |
	rs := source readStream.
	ws := target writeStream.
	missedIds := OrderedCollection new.
	entries := Dictionary new.
	
	[[rs atEnd] whileFalse: 
			[entry := self nextEntryFrom: rs.
			(entries at: entry key ifAbsentPut: [Set new]) add: entry].
	entries keys sorted do: 
			[:each |
			currentEntries := entries at: each.
			currentEntries size &gt; 1 ifTrue: [self halt].
			ws print: currentEntries any]]
			ensure: 
				[rs close.
				ws close].</body>

<body package="TranslationScanner" selector="convertFrom:to:">convertFrom: source to: target

	| rs ws entry |
	rs := source readStream.
	ws := target writeStream.
	missedIds := OrderedCollection new.
	
	[[rs atEnd] whileFalse: 
			[entry := self nextEntryFrom: rs.
			self updateEntry: entry.
			ws print: entry]]
			ensure: 
				[rs close.
				ws close].
	self reportIds: missedIds forFile: source.
	</body>

<body package="TranslationScanner" selector="missedId:">missedId: anID

	missedIds add: anID.</body>

<body package="TranslationScanner" selector="initialize">initialize
	"Initialize a newly created instance. This method must answer the receiver."

	super initialize.
	^self</body>

<body package="TranslationScanner" selector="convert:">convert: languageFolder
	"
	self new convert: '/Users/karsten/scratch/Documents/XcoDeZ/CuteClips3/German.lproj'
	"

	| langFolder folder enFolder stringsFiles |
	langFolder := languageFolder asFilename.
	folder := languageFolder asFilename head asFilename.
	enFolder := folder / 'English.lproj'.
	stringsFiles := langFolder directoryContents
				select: [:each | '*.strings' match: each].
	stringsFiles do: 
			[:each |
			((#('Localizable.strings') includes: each) not
				and: [(enFolder / each) exists])
					ifTrue: 
						[self readTranslationsFrom: enFolder / each.
						self convertFrom: langFolder / each
							to: langFolder
									/ ((each copyFrom: 1 to: (each lastIndexOf: $.)) , '_fixed.strings').
						self reportIds: originalEntries keys forFile: enFolder / each]]</body>

<body package="TranslationScanner" selector="nextEntryFrom:">nextEntryFrom: rs

	| comment cRS id key translation |
	rs upToAll: '/*'.
	comment := (rs upToAll: '*/'),'*/'.
	cRS := comment readStream.
	cRS skipSeparators.
	cRS upTo: $".
	self nextQuoteFrom: cRS.
	cRS upTo: $".
	self nextQuoteFrom: cRS.
	cRS upTo: $".
	id := self nextQuoteFrom: cRS.
	rs upTo: $".
	key := self nextQuoteFrom: rs.
	rs upTo: $".
	translation := self nextQuoteFrom: rs.
	rs
		upTo: Character cr;
		next.
	^(Entry new)
		comment: comment;
		id: id;
		key: key;
		translation: translation;
		yourself</body>

<body package="TranslationScanner" selector="readTranslationsFrom:">readTranslationsFrom: fileName

	| rs res |
	rs := fileName readStream.
	res := OrderedCollection new.
	[[rs atEnd] whileFalse: [res add: (self nextEntryFrom: rs)]]
		ensure: [rs close].
	originalEntries := Dictionary new.
	res do: [:each | originalEntries at: each id put: each]</body>

<body package="TranslationScanner" selector="reportIds:forFile:">reportIds: ids forFile: source

	ids notEmpty
		ifTrue: 
			[Transcript
				cr;
				show: 'MISSED IDs in:';
				show: source asString;
				cr].
	ids do: [:each | Transcript show: each]
		separatedBy: [Transcript show: ', ']</body>

<body package="TranslationScanner" selector="updateEntry:">updateEntry: anEntry

	| orgEntry |
	orgEntry := originalEntries at: anEntry id ifAbsent:[^self missedId: anEntry id].
	anEntry key: orgEntry key.
	originalEntries removeKey: anEntry id.</body>

<body package="TranslationScanner" selector="testQuoted">testQuoted
	"
	self new testQuoted
	"

	| data source goal |
	data := #('123" ' '123' '123\" "' '123" ' '123\\"' '123\').
	1 to: data size
		by: 2
		do: 
			[:i |
			source := data at: i.
			goal := data at: i + 1.
			(self nextQuoteFrom: source readStream) = goal ifFalse: [self halt]]</body>

<body package="TranslationScanner" selector="nextQuoteFrom:">nextQuoteFrom: rs

	| ws state c |
	ws := String new writeStream.
	state := #started.
	[rs atEnd or: [state = #done]] whileFalse: 
			[c := rs next.
			c = $\
				ifTrue: 
					[c := rs next.
					state := #special].
			(c = $" and: [state ~= #special])
				ifTrue: [state := #done]
				ifFalse: 
					[ws nextPut: c.
					state := #notDone]].
	^ws contents</body>

<body package="TranslationScanner" selector="filter:">filter: languageFolder
	"
	self new filter: '/Users/karsten/scratch/Documents/XcoDeZ/CuteClips3/French.lproj'
	"

	| langFolder stringsFiles |
	langFolder := languageFolder asFilename.
	stringsFiles := langFolder directoryContents
				select: [:each | '*.strings' match: each].
	stringsFiles do: 
			[:each |
			((#('Localizable.strings') includes: each) not)
					ifTrue: 
						[self filterFrom: langFolder / each
							to: langFolder
									/ ((each copyFrom: 1 to: (each lastIndexOf: $.)) , '_fixed.strings').
						]]</body>
</methods>

<methods>
<class-id>Converter</class-id> <category>accessing</category>

<body package="TranslationScanner" selector="originalEntries:">originalEntries: anObject
	originalEntries := anObject</body>

<body package="TranslationScanner" selector="originalEntries">originalEntries
	^originalEntries</body>
</methods>


<methods>
<class-id>Entry class</class-id> <category>instance creation</category>

<body package="TranslationScanner" selector="new">new
	"Answer a newly created and initialized instance."

	^super new initialize</body>
</methods>


<methods>
<class-id>Entry</class-id> <category>comparing</category>

<body package="TranslationScanner" selector="hash">hash

	^self key hash bitXor: self translation hash.</body>

<body package="TranslationScanner" selector="=">= anEntry

	^anEntry key = self key and: [anEntry translation = self translation]</body>
</methods>

<methods>
<class-id>Entry</class-id> <category>accessing</category>

<body package="TranslationScanner" selector="translation:">translation: anObject
	translation := anObject</body>

<body package="TranslationScanner" selector="comment:">comment: anObject
	comment := anObject</body>

<body package="TranslationScanner" selector="translation">translation
	^translation</body>

<body package="TranslationScanner" selector="key:">key: anObject
	key := anObject</body>

<body package="TranslationScanner" selector="id">id
	^id</body>

<body package="TranslationScanner" selector="comment">comment
	^comment</body>

<body package="TranslationScanner" selector="id:">id: anObject
	id := anObject</body>

<body package="TranslationScanner" selector="key">key
	^key</body>

<body package="TranslationScanner" selector="quote:">quote: aString

	| ws |
	ws := (String new: aString size + 2) writeStream.
	ws nextPut: $".
	aString do: 
			[:each |
			(#($\ $") includes: each) ifTrue: [ws nextPut: $\].
			ws nextPut: each].
	ws nextPut: $".
	^ws contents</body>
</methods>

<methods>
<class-id>Entry</class-id> <category>printing</category>

<body package="TranslationScanner" selector="printOn:">printOn: aStream

	aStream
		nextPutAll: comment;
		cr;
		nextPutAll: (self quote: key);
		nextPutAll: ' = ';
		nextPutAll: (self quote: translation);
		nextPut: $;;
		cr;
		cr</body>
</methods>

<methods>
<class-id>Entry</class-id> <category>initialize-release</category>

<body package="TranslationScanner" selector="initialize">initialize
	"Initialize a newly created instance. This method must answer the receiver."

	super initialize.
	" *** Edit the following to properly initialize instance variables ***"
	id := nil.
	key := nil.
	translation := nil.
	" *** And replace this comment with additional initialization code *** "
	^self</body>
</methods>



</st-source>
